#include "kernelsolver.h"

const int BLOCKSIZE = 16;
const double INF=1e9;

void __global__ Kernel(double *Matrix, int k, int n){
  int i = blockIdx.y * BLOCKSIZE + threadIdx.y;
  int j = blockIdx.x * BLOCKSIZE + threadIdx.x;

  if (i < n && j < n && Matrix[i*n + k] < INF && Matrix[k*n + j] < INF){
    Matrix[i*n+j] = min(Matrix[i*n + j], Matrix[i*n + k] + Matrix[k*n + j]);
  }
}

float solve(double *Matrix, int n){
  int dev_n = n * n * sizeof(double);
  double *dev_Matrix;

  cudaMalloc((void**) &dev_Matrix, dev_n);

  cudaMemcpy(dev_Matrix, Matrix, dev_n, cudaMemcpyHostToDevice);

  dim3 threads(BLOCKSIZE, BLOCKSIZE);
  dim3 grid(n/BLOCKSIZE, n/BLOCKSIZE);

  cudaEvent_t start, stop;
  float gpuTime = 0;

  /*------------------------------*/
  cudaEventCreate(&start);
  cudaEventCreate(&stop);

  cudaEventRecord(start, 0);

  REP(k,n) Kernel<<<grid, threads>>>(dev_Matrix, k, n);

  cudaEventRecord(stop, 0);
  cudaEventSynchronize(stop);
  cudaEventElapsedTime(&gpuTime, start, stop);
  /*------------------------------*/

  cudaMemcpy(Matrix, dev_Matrix, dev_n, cudaMemcpyDeviceToHost);

  cudaFree(dev_Matrix);
  cudaDeviceReset();

  return gpuTime;
}
